package yui.example.common.controls.renderer
{
	import flash.display.Graphics;
	import flash.events.MouseEvent;
	import flash.geom.Rectangle;
	
	import mx.charts.ChartItem;
	import mx.charts.chartClasses.GraphicsUtilities;
	import mx.charts.series.LineSeries;
	import mx.charts.series.items.LineSeriesItem;
	import mx.core.IDataRenderer;
	import mx.core.UIComponent;
	import mx.graphics.IFill;
	import mx.graphics.IStroke;
	import mx.graphics.SolidColor;
	import mx.styles.StyleManager;
	import mx.utils.ColorUtil;
	
	import yui.example.common.events.DataChangeEvent;
	
	
	public class CircleItemRenderer extends UIComponent implements IDataRenderer
	{
		private static var rcFill:Rectangle = new Rectangle();
	
		public function CircleItemRenderer() 
		{
			super();
		}
		
		protected var startY:Number;
	
		private var _chartItem:Object;
		
		private var isRollOver:Boolean = false;
	
		[Inspectable(environment="none")]
	
		public function get data():Object
		{
			return _chartItem;
		}
		
		public function set data( value:Object ):void
		{
			if ( _chartItem == value )
				return;
	
			_chartItem = value;

			addEventListener( MouseEvent.ROLL_OUT, rollOutHandler );
			addEventListener( MouseEvent.ROLL_OVER, rollOverHandler );
		}

		private function rollOutHandler( event:MouseEvent ):void
		{
			isRollOver = false;
			invalidateDisplayList();
		}

		private function rollOverHandler( event:MouseEvent ):void
		{
			isRollOver = true;
			invalidateDisplayList();
			addEventListener( MouseEvent.MOUSE_DOWN, mouseDownHandler );
		}
		
		private function mouseDownHandler( event:MouseEvent ):void
		{
			LineSeries(owner).owner.addEventListener(MouseEvent.MOUSE_MOVE,mouseMoveHandler);
			LineSeries(owner).owner.addEventListener(MouseEvent.MOUSE_UP,mouseUpHandler);

			startY = owner.mouseY;
		}
		
		private function mouseMoveHandler( event:MouseEvent ):void
		{
			var divY:Number = owner.mouseY- startY;
			startY = owner.mouseY;
			
			this.y += divY;
			
			var startData:Array = LineSeries(owner).dataTransform.invertTransform(owner.mouseX,owner.mouseY);
			var start1PixelData:Array = LineSeries(owner).dataTransform.invertTransform(owner.mouseX,owner.mouseY+1);

			start1PixelData[1] -= startData[1];
			
			var divYValue:Number = start1PixelData[1] * divY;
			var yfield:String = LineSeries(owner).yField;
			LineSeriesItem(data).item[ yfield ] += Math.ceil(divYValue);
			
			var dataChangeEvent:DataChangeEvent = new DataChangeEvent( DataChangeEvent.CHANGE, true );
			dataChangeEvent.xdata = start1PixelData[0];
			dataChangeEvent.ydata = start1PixelData[1];
			
			LineSeries(owner).dispatchEvent( dataChangeEvent );
		}
		
		private function mouseUpHandler( event:MouseEvent ):void
		{
			removeEventListener( MouseEvent.MOUSE_DOWN, mouseDownHandler );
			removeEventListener( MouseEvent.MOUSE_DOWN, mouseDownHandler );
			
			LineSeries(owner).owner.removeEventListener( MouseEvent.MOUSE_UP, mouseUpHandler );
			LineSeries(owner).owner.removeEventListener( MouseEvent.MOUSE_MOVE, mouseMoveHandler );
		}
	
		override protected function updateDisplayList( unscaledWidth:Number,
													  unscaledHeight:Number ):void
		{
			super.updateDisplayList( unscaledWidth, unscaledHeight );
			
			var fill:IFill;
			var state:String = "";
			
			if ( _chartItem is ChartItem && _chartItem.hasOwnProperty( 'fill' ) )
			{
			 	fill = _chartItem.fill;
			 	state = _chartItem.currentState;
			}
			else
			{
			 	fill = GraphicsUtilities.fillFromStyle(getStyle('fill'));
			} 
	        
	        var color:uint;
			var adjustedRadius:Number = 0;
			
			switch(state)
			{
				case ChartItem.FOCUSED:
				case ChartItem.ROLLOVER:
					if ( StyleManager.isValidStyleValue( getStyle( 'itemRollOverColor' ) ) )
					{
						color = getStyle( 'itemRollOverColor' );
					}
					else
					{
						color = ColorUtil.adjustBrightness2( GraphicsUtilities.colorFromFill( fill ), -20 );
					}
					
					fill = new SolidColor( color );
					adjustedRadius = getStyle( 'adjustedRadius' );
					if( !adjustedRadius )
					{
						adjustedRadius = 0;
					}	
					break;
				case ChartItem.DISABLED:
					if ( StyleManager.isValidStyleValue( getStyle( 'itemDisabledColor' ) ) )
					{
						color = getStyle('itemDisabledColor');
					}
					else
					{
						color = ColorUtil.adjustBrightness2( GraphicsUtilities.colorFromFill( fill ), 20 );
					}
					
					fill = new SolidColor( GraphicsUtilities.colorFromFill( color ) );
					
					break;
				case ChartItem.FOCUSEDSELECTED:
				case ChartItem.SELECTED:
					if ( StyleManager.isValidStyleValue( getStyle( 'itemSelectionColor' ) ) )
					{
						color = getStyle( 'itemSelectionColor' );
					}
					else
					{
						color = ColorUtil.adjustBrightness2( GraphicsUtilities.colorFromFill( fill ), -30 );
					}
					fill = new SolidColor( color );
					adjustedRadius = getStyle( 'adjustedRadius' );
					if( !adjustedRadius )
					{
						adjustedRadius = 0;
					}
					
					break;
			} 

			var stroke:IStroke = getStyle( "stroke" );
			var w:Number = stroke ? stroke.weight / 2 : 0;
			if( isRollOver )
			{
				w = 10;
			}
	
			rcFill.right = unscaledWidth;
			rcFill.bottom = unscaledHeight;
	
			var g:Graphics = graphics;
			g.clear();		
			if ( stroke )
			{
				stroke.apply( g );
			}
			
			if ( fill )
			{
				fill.begin( g, rcFill );
			}
			
			g.drawEllipse( w - adjustedRadius,
					w - adjustedRadius,
					unscaledWidth - 2 * w + adjustedRadius * 2, 
					unscaledHeight - 2 * w + adjustedRadius * 2);
	
			if ( fill )
			{
				fill.end( g );
			}
		}

	}
}